##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################
"""Base class for all PortalContent"""

__version__='$Revision: 1.45 $'[11:-2]

import string, urllib

import Globals
from DateTime import DateTime
from Globals import HTMLFile, Persistent
from AccessControl.Role import RoleManager
from OFS.SimpleItem import Item
from AccessControl.Permission import Permission

from ToolboxActionProvider import ToolboxActionProvider
from DublinCore import DublinCore

class PortalContent(Item, Persistent, RoleManager, DublinCore,
                    ToolboxActionProvider):
    """
    Base class for portal objects.
    
    Provides hooks for reviewing, indexing, and PTK UI.
    """

    review_history=()
    review_state='private'
    effective_date=None
    
    manage_options=(
        {'label':'View', 'action':'index_html'},
        {'label':'Security', 'action':'manage_access'},
        )
    
    __ac_permissions__=(
        ('View', ('SearchableText', 'index_html', ''),
         ('Owner','Manager','Reviewer')),
        ('Request review', ('set_effective_date'), ('Owner',)),
        )

    def getPortal(self):
        # Ensures we don't accidentally acquire a getPortal object that
        # returns incorrect info.  This is for both security and avoidance
        # of surprise errors in the presence of a getPortal object.
        if not hasattr(self, 'aq_parent'):
            return None
        parent = self.aq_parent
        while 1:
            portal = parent.getPortal()
            base = getattr(portal, 'aq_base', portal)
            if getattr(base, '_isAPortalObject', 0):
                return portal
            else:
                parent = parent.aq_parent
            

    # indexed methods
    # ---------------
    
    def SearchableText(self):
        "Returns a concatination of all searchable text"
        # Should be overriden by portal objects
        return "%s %s" % (self.Title(), self.Description())

    def allowedRolesAndUsers(self, permission='View'):
        """
        Return a list of roles and users with View permission.
        Used by PortalCatalog to filter out items you're not allowed to see.
        """
        allowed = []
        valid_roles = self.valid_roles()
        for p in self.ac_inherited_permissions(1):
            name, value = p[:2]
            if name == permission:
                p = Permission(name, value, self)
                allowed=p.getRoles() + ['Owner']
        localroles = self.__mergedLocalRoles()
        for user in localroles.keys():
            for role in localroles[user]:
                if role in allowed and not 'user:'+user in allowed:
                    allowed.append('user:'+user)
        return allowed
            
    # Support for catalog result permission checking

    def __mergedLocalRoles(self):
        "Returns a merging of my and my ancestors __ac_local_roles__"
        merged = {}
        object = getattr(self, 'aq_inner', self)
        while 1:
            if hasattr(object, '__ac_local_roles__'):
                dict=object.__ac_local_roles__ or {}
                if callable(dict): dict = dict()
                for k in dict.keys():
                    if merged.has_key(k):
                        merged[k] = merged[k] + dict.get(k)
                    else:
                        merged[k] = dict.get(k)
            if hasattr(object, 'aq_parent'):
                object=object.aq_parent
                continue
            if hasattr(object, 'im_self'):
                object=object.im_self
                object=getattr(object, 'aq_inner', object)
                continue
            break
        return merged


    # Reviewing methods
    # -----------------
    
    # Attributes:
    #   'review_state'   indicates state of object,
    #                    'published'|'private'|'pending'
    #   'review_history' is a list of dicts which describes a
    #                    history of reviewing actions
    #                    
    # Permissions:
    #
    # 'Review portal content'   permission to review an object for publishing
    # 'Request review'          permission to request an object be published
    
    publish=HTMLFile('dtml/publish',globals())
    
    def _review_action(self, action, review_state, comments, REQUEST):
        # Give portal policy a chance to modify the review action
        action,review_state,comments = self.getPortal().before_review(
            self, action, review_state, comments, REQUEST)
        
        rh=list(self.review_history)
        rh.append({
            'actor': self.getPortal().getAuthenticatedMember().getUserName(),
            'action': action,
            'review_state': review_state,
            'time': DateTime(),
            'comments': comments,
            })
        self.review_history=rh
        self.review_state=review_state

        # Ask the portal for any permission changes to the object
        set = self.getPortal().review_policy(self)
        #print set
        for role, grant, revoke in set:
            for p in self.ac_inherited_permissions(1):
                name, value = p[:2]
                p=Permission(name, value, self)
                if name in grant:
                    p.setRole(role, 1)
                elif name in revoke:
                    p.setRole(role, 0)

        self.reindex_object()

    def setReviewState(self, review_state, comments, effective_date, REQUEST):
        """
        Reset this item's review_state, and optionally set it's effective date.
        """
        user = self.getPortal().getAuthenticatedMember()
        message = ""
        if not (user.has_permission('Request review', self) or
                user.has_permission('Review portal content', self)):
            return self.publish(self, REQUEST, portal_status_message=
                                "Error: You do not have permission to change "\
                                "this object's status.")
        if review_state != self.review_state:
            if review_state == 'pending' and self.review_state == 'published':
                return self.publish(self, REQUEST, portal_status_message=
                                    "Error: Item already published.")
            if review_state == 'published' and not \
               (user.has_permission('Review portal content', self) or \
                (user.has_permission('Contribute portal content', self) and
                 user.has_role('Owner', self))):
                # The user is not allowed to review this item and the user
                # is not both a Contributor and the owner of this object.
                return self.publish(self, REQUEST, portal_status_message=
                                    "Error: You do not have permission to " \
                                    "publish this item")
            if review_state != 'published' and \
               not user.has_role('Owner', self) and \
               not comments:
                return self.publish(self, REQUEST, portal_status_message=
                                    "Error: Must specify reason in comment.")
            self._review_action('made ' + review_state,
                                review_state,
                                comments,
                                REQUEST)
            message = "Made item " + self.review_state + ".  "
        if effective_date:
            try:
                date = DateTime(effective_date)
                if self.effective_date != date:
                    self.effective_date = date
                    message = message + "Effective date set."
            except:
                message = message + "Could not parse effective date, " \
                                    "date not set."
        return self.publish(self, REQUEST, portal_status_message=message)

    # Effective date methods
    # ---------------------

    # If effective date is set then an object will not be published
    # until its effective date. This is enforced in the
    # standard_html_header

    def set_effective_date(self, effective_date, REQUEST):
        """
        Set the effective_date property, which is when this resource becomes
        available
        """
        try:
            date = DateTime(effective_date)
        except:
            return self.publish(
                self, REQUEST, portal_status_message=
                "Could not parse effective date.  Please try again.")
        self.effective_date = date
        return self.publish(
            self, REQUEST, portal_status_message="Effective date set.")
            

    # Cataloging methods
    # ------------------

    # Catalog comes from acquired 'getPortalCatalog' method,
    # which should be implemented by the PortalObject.


    def index_object(self):
        catalog = self.getPortal().getCatalog()
        if catalog is not None:
            catalog.catalog_object(
                self, urllib.unquote('/' + self.absolute_url(1)))

    def unindex_object(self):
        catalog = self.getPortal().getCatalog()
        if catalog is not None:
            catalog.uncatalog_object(
                urllib.unquote('/'+self.absolute_url(1)))

    def reindex_object(self):
        self.unindex_object()
        self.index_object()
        
    def manage_afterAdd(self, item, container):
        "Tell the SiteIndex about my glorious birth."
        self.index_object()

    def manage_beforeDelete(self, item, container):
        "Inform the SiteIndex about my eminent demise."
        self.unindex_object()


    # PTK interface methods
    # ---------------------
    
    def toolbox_actions(self, url=0):
        # Report toolbox actions applicable to all kinds of content
        if url==0: url=self.absolute_url()
        return (
            {'name':'View',
             'url': url + '/index_html',
             'permissions': ['View'],
             'category': 'object'
             },
            {'name':'Set status',
             'url': url + '/publish',
             'permissions': ['Request review', 'Review portal content'],
             'category': 'object'
             },
            )

    # Useful DTML methods
    # -------------------
    
    byline = HTMLFile('dtml/byline', globals())



Globals.default__class_init__(PortalContent)
