##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################
"""Portal object base classes"""

__version__='$Revision: 1.72 $'[11:-2]

import sys, os, string, time, random
from urllib import quote, unquote
from base64 import encodestring

from DateTime import DateTime
from DocumentTemplate import HTML
from App.Dialogs import MessageDialog
import OFS.SimpleItem
from Globals import HTMLFile
import Globals

from DublinCore import DublinCore
from Toolbox import Toolbox
from ToolboxActionProvider import ToolboxActionProvider
import PortalFolder
import MemberFolder
from ExtensionClass import Base


try:
    # Zope 2.2.x method.
    from AccessControl import getSecurityManager
    def getAuthenticatedUser(self):
        return getSecurityManager().getUser()
except:
    # Zope 2.1.x method.
    from AccessControl import User
    def getAuthenticatedUser(self):
        u = self.REQUEST.get('AUTHENTICATED_USER', None)
        if u is None:
            u = User.nobody
        return u


class MemberDataAccess(Base):

    # This class accesses member data through an acl_users object.
    
    def getAuthenticatedMember(self):
        '''
        Returns the currently authenticated member object or the Anonymous User.
        '''
        u = getAuthenticatedUser(self)
        if u is not None and not hasattr(u, 'aq_base'):
            u = u.__of__(self.acl_users)
        return u

    def getMemberById(self, id):
        '''
        Returns the given member.
        '''
        u = self.acl_users.getUser(id)
        if u is not None and not hasattr(u, 'aq_base'):
            u = u.__of__(self.acl_users)
        return u

    def addMemberFolder(self, REQUEST=None):
        """Add a member folder to a portal site object.

        This is done during initialization."""
        
        if hasattr(self.aq_base, 'acl_users'):
            return MessageDialog(
                   title  ='Item Exists',
                   message='This object already contains a User Folder',
                   action ='%s/manage_main' % REQUEST['URL1'])
        
        dest=self.this()
        ob=self.MembersClass()
        if hasattr(dest, '_setObject'):
            dest._setObject('acl_users', ob)
            dest.__allow_groups__=dest.acl_users
        else:
            dest.__allow_groups__=dest.acl_users=ob
            
        if REQUEST: return dest.manage_main(dest, REQUEST, update_menu=1)

    def addMember(self, name, password, roles, domains):
        self.acl_users._addUser(name, password, password, roles, domains)

    def _changeMember(self, user, name, password, roles, domains):
        # Can be overridden.
        # There is no standard for accomplishing this,
        # so for now try all the ways we know. :-(
        if hasattr(self.acl_users, '_changeUser'):
            self.acl_users._changeUser(name,
                                       password,
                                       password, # confirm
                                       roles,
                                       domains)
        elif hasattr(user, 'changeUser'):
            user.changeUser(password, roles, domains)
        if hasattr(self.acl_users, 'credentialsChanged'):
            # Take advantage of an interface provided by LoginManager.
            self.acl_users.credentialsChanged(user, name, password)


class PortalObjectBase(PortalFolder.PortalFolder, DublinCore,
                       ToolboxActionProvider, MemberDataAccess):

    meta_type='Portal Site'
    _isAPortalObject = 1

    _siteBase=sitePath=''

    __ac_roles__=('Member', 'Reviewer', 'Contributor')
    
    __ac_permissions__=(
        ('View management screens',
         ('portal_cut', 'portal_copy', 'portal_paste', 'portal_rename',
          'portal_delete')),
        ('Manage portal',
         ('addMemberFolder','addMember'), ('Owner',)),
        ('Contribute portal content', (), ('Contributor',)),
        )


    def register(self, username='', full_name='', company='', email='',
                 mail_me=0, REQUEST=None):
        """ """
        register_html=getattr(self, 'join_form')
        if not username or not email:
            return register_html(self, REQUEST,
                   error=('You must enter a value for Name and a valid Email '
                          'Address.'))
    
        user=self.getMemberById(username)
        if user is not None:
            return register_html(self, REQUEST,
                   error=('The login name you selected is already in use. '
                          'Please choose another.'))
        password = REQUEST.get('password', gen_password())
        disapproval = self.password_policy(password)
        if disapproval:
            return register_html(self, REQUEST,
                   error=(disapproval))
        confirm = REQUEST.get('confirm', password)
        if confirm != password:
            return register_html(self, REQUEST,
                   error=('Your password and confirmation did not match. '
                          'Please try again.'))
    
        # Add the member
    
        self.addMember(username, password, ['Member'], [])
    
        user=self.getMemberById(username)
        user.setMemberProperties(REQUEST)

        self.register_hook(REQUEST)
        
        # Generate email notification
        # XXX This breaks the division between the PTK and the DemoPortal.
        #     Must move properties from propertysheet to PortalObject.
        if getattr(self, 'validate_email', 0) or mail_me:
            self.registered_notify(self, REQUEST, password=password)
        
        return getattr(self, 'registered')(self, REQUEST)            

    def register_hook(self, *args): pass

    def password_policy(self, candidate):
        """Implement here the password characteristics policy for your site.
    
        Returns an error message on failure, or None if all is ok."""
        if len(candidate) < 5:
            return 'Your password must contain at least 5 characters.'


    def mail_password(self, forgotten_username, REQUEST):
        """Email a forgotten password to a member"""
    
        user = self.getMemberById(forgotten_username)
        if user is None:
            return MessageDialog(
                   title  ='Username not found',
                   message='The username you entered could not be found.',
                   action ='mail_password_form')
    
        return self.mail_password_template(
            self, REQUEST,
            # full_name=user.full_name,
            email=user.email,
            password=user._getPassword()
            )

    def manage_editSiteBase(self, base='', path='', REQUEST=None):
        """ """
        self._siteBase = base
        self._sitePath = path
        if REQUEST is not None:
            return self.manage_main(self, REQUEST)



    if 0:  # This is legacy code to deal with virtual hosts.

      def __bobo_traverse__(self, REQUEST, name=None):
        """ When I site object is traversed, it should reset the
        REQUESTs URL base.  The exception is when the environment
        variable SERVER_URL does not match the new base, in this case, 
        no rewriting should be done (otherwise, your object may become 
        unreachable if your virtual hosts stop working)"""

        if REQUEST.has_key('SERVER_URL') and \
           REQUEST['SERVER_URL'] == self._siteBase:
            REQUEST.setSite(self._siteBase)
            # the virtual site's PARENTS should end at the site object
            del REQUEST['PARENTS'][:]

        try: return getattr(self, name)
        except AttributeError: pass
        try: return self[name]
        except KeyError: pass
        method=REQUEST.get('REQUEST_METHOD', 'GET')
        if not method in ('GET', 'POST'):
            return NullResource(self, name, REQUEST).__of__(self)

        REQUEST.RESPONSE.notFoundError("%s\n%s" % (name, method))


      def absolute_url(self, relative=None):
        """ since absolute_url now recurses up the containment
        heirarchy, we want to return an empty string to indicate that
        the recursion should stop here because we are the virtual root 
        for this site object
        """
        # argh, I guess I should aquire REQUEST

        if hasattr(self, 'REQUEST') and \
           self.REQUEST['SERVER_URL'] == self._siteBase:
            if not relative:
                return self._siteBase
            else:
                return ''
        else:
            id=quote(self.id)

            p=getattr(self,'aq_inner', None)
            if p is not None: 
                url=p.aq_parent.absolute_url(relative)
                if url: id=url+'/'+id

            return id

      # END LEGACY CODE


    def changePassword(self, user, password, confirm, domains, REQUEST,
                       redirect=None):
        ' '
        auser=self.getAuthenticatedMember()
        if auser.getUserName() != user:
            raise 'Unauthorized'
        if password != confirm: raise 'Passwords don\'t match'
        self._changeMember(auser, user, password, auser.getRoles(), domains)

        if redirect is not None:
            REQUEST.RESPONSE.redirect(redirect)
        else:
            return self.personalize_form(
                self, REQUEST, portal_status_message="Password changed")

    # Action Provider interface

    def toolbox_actions(self, url=0):
        if url==0: url=self.absolute_url()
        try:
            user=self.getAuthenticatedMember()
            username=user.getUserName()
        except:
            username='Anonymous User'
            
        if username=='Anonymous User':
            actions = (
                {'name': 'Log in',
                 'url': url + '/login_form',
                 'permissions': [],
                 'category': 'user'},
                {'name': 'Join',
                 'url': url + '/join_form',
                 'permissions': [],
                 'category': 'user'},
                )
        else:
            actions = (
                {'name': 'Preferences',
                 'url': url + '/personalize_form',
                 'permissions': [],
                 'category': 'user'},
                {'name': 'Log out',
                 'url': url + '/logout',
                 'permissions' : [],
                 'category': 'user'},
                {'name': 'Undo',
                 'url': 'portal_undoForm',
                 'permissions': [],
                 'category': 'global'},
                {'name': 'Reconfigure portal',
                 'url': url + '/portal_config',
                 'permissions': ['Manage portal'],
                 'category': 'global'},
                )
        return actions
    
    # Reviewing Policy interface

    def notify_content(self, changed_object, user):
        # changed_object has just been modified by user.
        # Default policy doesn't care
        changed_object.reindex_object()

    def before_review(self, changed_object, action, review_state, comments,
                      REQUEST):
        # This hook gives the portal a chance to tweak the publishing request 
        return action, review_state, comments

    def review_policy(self, content):
        # The review_state of 'content' has just changed.  Tell the PTK what
        # the new permissions should be.
        if content.review_state == 'private':
            # Revoke 'View' from Anonymous and Reviewer
            return (
                ('Anonymous', (), ('View',)),
                ('Reviewer', (), ('View',)),
                )
        elif content.review_state == 'pending':
            # Give Reviewer 'View', revoke from Anonymous
            return (
                ('Anonymous', (), ('View',)),
                ('Reviewer', ('View',), ()),
                )
        elif content.review_state == 'published':
            # Give Anonymous 'View'
            return (
                ('Anonymous', ('View',), ()),
                )
        raise ValueError, "Unknown state: " + `content.review_state`
        
        
    # Undoing interface

    portal_undoForm = HTMLFile('dtml/portal_undoForm',globals())

    def portal_undo(self, transaction_info, REQUEST=None):
        """Perform undo and present portal interface
        """
        self.manage_undo_transactions(transaction_info)

        if REQUEST is not None:
            return self.portal_contents(
                self, REQUEST, portal_status_message='Transaction undone')

    def portal_undoable_transactions(self,
                                     first_transaction=None,
                                     last_transaction=None,
                                     PrincipiaUndoBatchSize=None,
                                     REQUEST=None):
        '''Lists the transactions this user is allowed to undo.
        '''
        user = self.getAuthenticatedMember()
        # arg list for undoable_transactions() changed in Zope 2.2.
        transactions = self.undoable_transactions(
            first_transaction=first_transaction, last_transaction=last_transaction,
            PrincipiaUndoBatchSize=PrincipiaUndoBatchSize)
        if user and not user.has_role('Manager', self):
            # Filter out transactions done by other members of the portal.
            transactions = filter(
                lambda x, user=user:
                    string.split(x['user_name'])[-1] == user.getUserName(),
                transactions
                )
        return transactions

    # Resource locating methods

    def getCatalog(self):
        "Return the portal's catalog"
        # This is potentially flakey because this class does not yet have
        # a way to initialize resources.  We must depend on the application to
        # create 'SiteIndex'.  XXX
        return getattr(self, 'SiteIndex', None)

    def getPortal(self):
        "Return the portal"
        return self

    def getToolbox(self, REQUEST):
        "Return a toolbox"
        return Toolbox().__of__(self)

    def portal_url(self):
        "Return the URL of the portal object, in the client's reckoning."
        # XXX
        return self.absolute_url()

Globals.default__class_init__(PortalObjectBase)


def gen_password(choose=random.choice,
                 chars=string.letters+string.digits, join=string.join):
    result=[]
    for n in range(5):
        result.append(choose(chars))
    return join(result, '')
    
