class PortalPolicy:
    """
    This interface declares hooks which can be used to implement
    matters if oirtal policy.

    Additional hooks are likely to appear.

    No Portal classes implement these hooks.  PortalObject defines
    no-op or 'reasonable default' versions.
    """

    def register_hook(self, *args):
        """
        Hook called by PortalObject.register once a new Member has
        been successfully added to the portal.  This gives a
        PortalObject subclass the opportunity to set up any
        application-specific resources the Member will require.

        register_hook is presently called with the REQUEST variable as
        it's only argument.  Perhaps this should be augmented.

        TODO: *args should definately be replaced with a proper signature!!
              There should be a permission assigned to this.
        
        Return: ignored
        Permission: None assigned.
        """

    def registered_notifty(self, REQUEST, password):
        """

        This hook is expected to mail or otherwise notify the user of
        their initial password.  I suspect this should be done by
        register_hook, and will probably go away soon.

        Return: ignored
        Permission: None assigned.
        """
        
    def password_policy(self, candidate):
        """
        Implement here the password characteristics policy for your
        site.  'candidate' is the password the user has selected.  You
        may wish to ensure that the password meets minimum length
        requirements, is not a dictionary word, contains
        non-alphabetic characters, etc.
    
        Return: error message on failure, or None if all is ok.
        Permission: None assigned.
        """

    def notify_content(self, changed_object, user):
        """

        This hook is called to notify the portal whenever some portal
        content has been changed.  'user' is the user who performed
        the change.  This hook gives the portal an opportunity to
        change the review_state of the object (so that changes require
        a review).  It is also required to reindex the object whether
        it makes any changes to it or not.  Theoretically, this method
        could veto the change altogether by raising an excpetion.  In
        this case, the exception's value should be the page you desire
        returned to the user.

        Return: Ignored.
        Permission: None assigned.
        """

    def before_review(self, changed_object, action, review_state, comments,
                      REQUEST):
        """

        This hook is called when a user make any request to change an
        object's review_state (that is, change it's published status).
        This give the portal an opportunity to change the actual
        reviewing action that is performed, add commentary, trigger
        email notification, etc.

        'changed_object' is the object for which the review_state is
        potentially being changed.

        'action' is a plain-english string describing the action being
        performed on the action.  Examples: 'item approved', 'item
        rejected', 'item published', etc.

        'review_state' is the state the user desires to place the
        changed_object in.  The PTK defines 'private', 'pending' and
        'published'.  This could potentially be extended, but this has
        not been examined in any detail.
        
        'comments' is an optional note attached to the action by the
        user.  This may be appended to, but the user's comments should
        not be changed.

        Return: A tuple of the desired action, review_state and
        comments, in that order.

        Permissions: None assigned.
        """

    def review_policy(self, content):
        """

        This hook is called when 'content' (a Portal Content object)
        has had it's review_state changed.  It determines what
        permissions should be granted to or revoked from various roles
        as a result of the change.

        This hook should return a sequence with one item per role to
        modify.  Each item should be a sequence containing the role
        name, a sequence of permissions to grant to the role, and a
        sequence of permissions to revoke from the role.

        For example, to make an item invisible to 'Anonymous' and
        visible to 'Reviewer', return this:

          (('Anonymous', (), ('View',)), ('Reviewer', ('View',), ()))

        A better example is found in the PTK's default policy, found
        in ZopePTK/PTKBase/PortalObject.py.
        
        """
