class PortalUI:
    """

    This interface defines the user interface elements used by the
    PTK.  This includes forms, mail templates, reports, and other DTML
    objects.

    These methods are not implemented by the PTK, but the PTK calls
    them.  They should be implemented by PTK applications.
    
    A lot of these methods are inconsistently named.  Some are
    '_notify', while others are '_template', and so forth.  This
    should be cleaned up.
    
    This documentation format doesn't always map so well to DTML
    objects...
    """

    def standard_html_header(self, REQEUST):
        """

        These requirements are shared between standard_html_header and
        standard_html_footer.

        The PTK and application elements will often pass DTML objects
        a string called 'portal_status_message'.  This message should
        be prominently displayed somewhere if it exists.

        The header or footer is expected to display the Portal
        Toolbox.  This is done with the following DTML snippet:

        <dtml-var "getPortal().getToolbox(REQUEST)(this(), REQUEST)">

        The standard header is most unfortunately responsible for
        preventing unprivilaged users from seeing future-dated items.
        Here's an example of how this can be implemeneted:

<dtml-if "_.hasattr(this(),'Date') and Date().isFuture()">
 <dtml-let member="getPortal().getAuthenticatedMember()">
 <dtml-unless "member.has_permission('Request review',this())
   or member.has_permission('Review portal content',this())">
  <dtml-var "RESPONSE.unauthorized()">
 </dtml-unless>
</dtml-if>

        If anyone can provide a solution which does NOT depend on the
        header, it would be a huge boon.
        """

    def join_form(self, REQUEST):
        """

        This is the form the user fills out to become a Member.
        Before presenting the form, this page should verify that the
        user is not already logged on as a user.

        This form should collect at minimum: username, full_name, and
        email.  The form may additionally collect any information
        defined on the User class's property sheets.

        The form should prominently display the contents of 'error' if
        this variable has been passed to it.  This will contain error
        messages from the code which validates the user input from
        this form.  The form should check REQEUST.form to see if the
        user has already supplied any of these form values on a
        previous attempt, and pre-fill the form with them if so.

        The action of this form should be 'register'.
        """

    def logged_in(self, REQUEST):
        """

        This page is the target of the login form.
        This page should check who is viewing it.  If it's the
        Anonymous User, it should display a "Sorry, try again" style
        message, perhaps with pointers to "Join" and "I forgot my
        password".  Otherwise, this page should display a welcome
        message, and perform whatever updates to the user object are
        necessary each log-in.

        Since this page is also displayed when login has failed,
        perhaps it should be renamed to simply 'login'.
        """

    def logged_out(self, REQUEST):
        """

        This page is served to the user after they've been logged
        out.  Anything necessary to perform the log out opperation
        will already have been done, this is just a status message.
        """

    def login_form(self, REQUEST):
        """

        This page should present the login form.  The action of the
        form should be "&dtml.url-logged_in".  The information this
        form should collect is application-dependant.
        """

    def mail_password_form(self, REQUEST):
        """

        This form is used by the user to recover a lost or forgotten
        password.  It should collect a value for
        'forgotten_username' and submit it to 'mail_password'.

        Perhaps 'forgotten_username' should be renamed to
        'identifier', since a particular implementation may want to
        use an email address or some other unique piece of information
        more difficult to forget than a username.
        """

    def mail_password_template(self, REQEUST, password, email):
        """

        This page should contain a sendmail tag which, using the
        MailHost named 'MailHost', should send 'password' to the
        address 'email'.  The portal's email address and name are
        available as 'email_from_address' and 'email_from_name'.

        The result of calling this method is returned to the user, so
        it should also provide a message indicating that the mail has
        been sent successfully.
        """

    def password_form(self, REQEUST):
        """

        This is the form the user uses to change their password and
        domains.  It should collect values for 'password', 'confirm'
        ("type your password again to confirm"), and
        'domains:tokens'.  Additionally, it should provide the user's
        username in a hidden input called 'user'.  The action of this
        form should be 'changePassword'.

        I suppose changePassword could be made smart enough to deduce
        the username if it is not explicitly provided.
        """

    def personalize_form(self, REQEUST):
        """

        This is the form the user uses to edit the non-security
        related portions of their user object.  In user terms, to edit
        their profile and preferences.  The action of this form should
        be 'personalize', which essentialy just called
        setMemberProperties to save the new values.
        """

    def registered(self, REQUEST):
        """

        This page is served to the user upon seccessful registration.
        It should notify the user that their password has been emailed
        to them (or whatever the mechanism is).
        """

    def registered_notify(self, REQUEST, username, password, email):
        """
        This page is used to to notify the user of the creation of
        their account.  It is passed the selected username, the
        generated password, and the given email address.  Since the
        return value of this method is discarded, it should generate
        some external notificaiton, typically via email.

        See mail_password_template above for information about sending
        email via the portal.
        """
